/*
 This file is part of Subsonic.

 Subsonic is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Subsonic is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Subsonic.  If not, see <http://www.gnu.org/licenses/>.

 Copyright 2009 (C) Sindre Mehus
 */

package net.sourceforge.subsonic.androidapp.activity;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.TextView;
import android.view.KeyEvent;
import android.util.Log;
import net.sourceforge.subsonic.androidapp.service.DownloadService;
import net.sourceforge.subsonic.androidapp.service.DownloadServiceImpl;
import net.sourceforge.subsonic.androidapp.util.Constants;
import net.sourceforge.subsonic.androidapp.util.MergeAdapter;
import net.sourceforge.subsonic.androidapp.util.Util;
import net.sourceforge.subsonic.u1m.R;

import net.sourceforge.subsonic.u1m.androidapp.Analytics;
import com.google.android.apps.analytics.GoogleAnalyticsTracker;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;

public class MainActivity extends SubsonicTabActivity {
    private static final String TAG = MainActivity.class.getSimpleName();

    private static final int MENU_GROUP_SERVER = 10;
    private static final int MENU_ITEM_SERVER_1 = 101;
    private static final int MENU_ITEM_SERVER_2 = 102;
    private static final int MENU_ITEM_SERVER_3 = 103;
    private static final int MENU_ITEM_OFFLINE = 104;

    private static boolean infoDialogDisplayed;

    private long startupTime;

    /**
     * Called when the activity is first created.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        this.startupTime = System.currentTimeMillis();

        this.tracker.trackPageView(TAG);

        PreferenceManager.setDefaultValues(this, R.xml.settings, false);
        PreferenceManager.setDefaultValues(this, R.xml.debug_settings, true);

        setContentView(R.layout.main);

        View buttons = LayoutInflater.from(this).inflate(R.layout.main_buttons, null);

        final View serverButton = buttons.findViewById(R.id.main_select_server);
        final View shuffleButton = buttons.findViewById(R.id.main_shuffle);
        final View settingsButton = buttons.findViewById(R.id.main_settings);
        final View debugSettingsButton = buttons.findViewById(R.id.main_debug_settings);
        final View dummyView = findViewById(R.id.main_dummy);
        final View albumsTitle = buttons.findViewById(R.id.main_albums);
        final View albumsNewestButton = buttons.findViewById(R.id.main_albums_newest);
        final View albumsRandomButton = buttons.findViewById(R.id.main_albums_random);
/*  Not yet supported by Ubuntu One server.
        final View albumsHighestButton = buttons.findViewById(R.id.main_albums_highest);
        final View albumsRecentButton = buttons.findViewById(R.id.main_albums_recent);
        final View albumsFrequentButton = buttons.findViewById(R.id.main_albums_frequent);
*/
        final TextView serverTextView = (TextView) serverButton.findViewById(R.id.main_select_server_2);

        int instance = Util.getActiveServer(this);
        String name = Util.getServerName(this, instance);
        serverTextView.setText(name);

        ListView list = (ListView) findViewById(R.id.main_list);

        MergeAdapter adapter = new MergeAdapter();
        adapter.addViews(Arrays.asList(serverButton, shuffleButton, settingsButton), true);
        File subsonicDir = new File(Environment.getExternalStorageDirectory(), "subsonic");
        File debugFlagFile = new File(subsonicDir, "showdebugmenu");
        if (debugFlagFile.exists()) {
            Log.d(TAG, "Revealing debug settings button.");
            adapter.addView(debugSettingsButton, true);
        } else {
            Log.d(TAG, "Hiding debug settings button.");
        }
        if (!Util.isOffline(this)) {
            this.tracker.trackEvent("Life", "Startup", "isonline", 1);
            adapter.addView(albumsTitle, false);
            adapter.addViews(Arrays.asList(albumsNewestButton, albumsRandomButton), true);
/*  Not yet supported by Ubuntu One server.
            adapter.addViews(Arrays.asList(albumsHighestButton, albumsRecentButton, albumsFrequentButton), true);
*/
        } else {
            this.tracker.trackEvent("Life", "Startup", "isoffline", 1);
        }
        list.setAdapter(adapter);
        registerForContextMenu(dummyView);

        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if (view == serverButton) {
                    dummyView.showContextMenu();
                } else if (view == settingsButton) {
                    startActivity(new Intent(MainActivity.this, SettingsActivity.class));
                } else if (view == debugSettingsButton) {
                    startActivity(new Intent(MainActivity.this, DebugSettingsActivity.class));
                } else if (view == shuffleButton) {
                    Intent intent = new Intent(MainActivity.this, DownloadActivity.class);
                    intent.putExtra(Constants.INTENT_EXTRA_NAME_SHUFFLE, true);
                    Util.startActivityWithoutTransition(MainActivity.this, intent);
                } else if (view == albumsNewestButton) {
                    showAlbumList("newest");
                } else if (view == albumsRandomButton) {
                    showAlbumList("random");
/*  Not yet supported by Ubuntu One server.
                } else if (view == albumsHighestButton) {
                    showAlbumList("highest");
                } else if (view == albumsRecentButton) {
                    showAlbumList("recent");
                } else if (view == albumsFrequentButton) {
                    showAlbumList("frequent");
*/
                }
            }
        });

    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View view, ContextMenu.ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, view, menuInfo);

        MenuItem menuItem1 = menu.add(MENU_GROUP_SERVER, Util.SERVER_ID_UBUNTUONE_CUSTOMER, 1, Util.getServerName(this, Util.SERVER_ID_UBUNTUONE_CUSTOMER));
        MenuItem menuItem2 = menu.add(MENU_GROUP_SERVER, Util.SERVER_ID_UBUNTUONE_DEMO, 3, Util.getServerName(this, Util.SERVER_ID_UBUNTUONE_DEMO));
        MenuItem menuItem4 = menu.add(MENU_GROUP_SERVER, Util.SERVER_ID_OFFLINE, 2, Util.getServerName(this, Util.SERVER_ID_OFFLINE));
        menu.setGroupCheckable(MENU_GROUP_SERVER, true, true);

        switch (Util.getActiveServer(this)) {
            case 0:
                menuItem4.setChecked(true);
                break;
            case 1:
                menuItem1.setChecked(true);
                break;
            case 2:
                menuItem2.setChecked(true);
                break;
//            case MENU_ITEM_SERVER_3:
//                menuItem3.setChecked(true);
//                break;
        }
    }

    @Override
    public boolean onContextItemSelected(MenuItem menuItem) {
        DownloadService service = getDownloadService();
        this.tracker.trackEvent("UI", "Usage", "contextMenu", menuItem.getItemId());

        int nextServerId = menuItem.getItemId();

        if (Util.getActiveServer(this) != nextServerId) {
            if (service != null) {
                service.clear();
            }
        }

        if (nextServerId == Util.SERVER_ID_UBUNTUONE_CUSTOMER) {
            if (! hasCredentialsStored()) {
                showInfoDialog();
                return true;
            }
        }

        setActiveServer(nextServerId);

        restart();
        return true;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onPrepareOptionsMenu(menu);
        MenuItem mi;
        mi = menu.add(Menu.NONE, 3, Menu.NONE, R.string.main_settings);
        mi.setIcon(R.drawable.main_settings);

        mi = menu.add(Menu.NONE, 1, Menu.NONE, R.string.main_menu_help_label);
        mi.setIcon(R.drawable.main_help);

        mi = menu.add(Menu.NONE, 4, Menu.NONE, R.string.main_menu_demo_music_label);
        mi.setIcon(R.drawable.main_help);

        mi = menu.add(Menu.NONE, 2, Menu.NONE, R.string.main_menu_exit_label);
        mi.setIcon(R.drawable.main_exit);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        this.tracker.trackEvent("UI", "Usage", "optionsMenu", 1);
        switch (item.getItemId()) {
            case 3:
                startActivity(new Intent(MainActivity.this, SettingsActivity.class));
                return true;
            case 1: {
                Intent intent = new Intent(MainActivity.this, HelpActivity.class);
                startActivity(intent);
                return true;
            }
            case 2: {
                stopService(new Intent(MainActivity.this, DownloadServiceImpl.class));
                this.tracker.trackEvent("Life", "Shutdown", "diesBy finish", 1);
                finish();
                return true;
            }
            case 4: {
                Intent intent = new Intent(MainActivity.this, HelpActivity.class);
                intent.putExtra(HelpActivity.INTENT_KEY_URL, getResources().getString(R.string.demo_music_url));
                startActivity(intent);
                return true;
            }
		}
        return super.onOptionsItemSelected(item);
	}

    private void setActiveServer(int instance) {
        this.tracker.trackEvent("UI", "Usage", "settingActiveServer", 1);
        if (Util.getActiveServer(this) != instance) {
            DownloadService service = getDownloadService();
            if (service != null) {
                service.clear();
            }
            Util.setActiveServer(this, instance);
        }
    }

    private void restart() {
        Intent intent = new Intent(this, MainActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        Util.startActivityWithoutTransition(this, intent);
    }


    public boolean hasCredentialsStored() {
        SharedPreferences prefs = getSharedPreferences(Constants.PREFERENCES_FILE_NAME, 0);
        String username = prefs.getString(Constants.PREFERENCES_KEY_USERNAME + 1, null);
        String password = prefs.getString(Constants.PREFERENCES_KEY_PASSWORD + 1, null);

        return ((Util.trimToNull(username) != null) && (Util.trimToNull(password) != null));
    }

    private void showInfoDialog() {
        if (Util.isOffline(this)) {
            return;
        }

        if (! hasCredentialsStored()) {
            Util.requestAuthentication(this);
        }
    }

    private void showAlbumList(String type) {
        Intent intent = new Intent(this, SelectAlbumActivity.class);
        intent.putExtra(Constants.INTENT_EXTRA_NAME_ALBUM_LIST_TYPE, type);
        intent.putExtra(Constants.INTENT_EXTRA_NAME_ALBUM_LIST_SIZE, 20);
        intent.putExtra(Constants.INTENT_EXTRA_NAME_ALBUM_LIST_OFFSET, 0);
        Util.startActivityWithoutTransition(this, intent);
    }

    @Override
    public void onPause() {
        this.tracker.trackEvent("Life", "Shutdown", "diesBy pause", 1);
        super.onPause();
    }

    @Override
    public void onStop() {
        this.tracker.trackEvent("Life", "Shutdown", "diesBy pauseandstop", 1);
        super.onStop();
    }

    @Override
    public void onDestroy() {
        this.tracker.trackEvent("Life", "Shutdown", "diesBy pauseandstopanddestroy", 1);
        this.tracker.trackEvent("Life", "Shutdown", "ageSeconds", (int) ((System.currentTimeMillis() - this.startupTime) / 1000));
        super.onDestroy();
    }
}
